// $Id: SDLFont.cc,v 1.8 2002/09/26 17:13:08 m9710797 Exp $

/*  Written By: Garrett Banuk <mongoose@mongeese.org>
 *  This is free, just be sure to give me credit when using it
 *  in any of your programs.
 */

#ifdef HAVE_SDL_IMAGE_H
#include "SDL_image.h"
#else
#include "SDL/SDL_image.h"
#endif

#include "SDLFont.hh"
#include "openmsx.hh"
#include "MSXException.hh"

const int NUM_CHRS = 256;
const int CHARS_PER_ROW = 16;
const int CHARS_PER_COL = NUM_CHRS / CHARS_PER_ROW;

SDLFont::SDLFont(const std::string &bitmapName)
{
	// load the font bitmap
	SDL_Surface *tempSurface;
	if (!(tempSurface = IMG_Load(bitmapName.c_str())))
		throw MSXException("Can't load font");

	fontSurface = SDL_DisplayFormat(tempSurface);
	SDL_FreeSurface(tempSurface);

	charWidth  = fontSurface->w / CHARS_PER_ROW;
	charHeight = fontSurface->h / CHARS_PER_COL;

	// Set font as transparent. The assumption we'll go on is that the
	// first pixel of the font image will be the color we should treat
	// as transparent.
	byte r = ((byte*)fontSurface->pixels)[0];
	byte g = ((byte*)fontSurface->pixels)[1];
	byte b = ((byte*)fontSurface->pixels)[2];
	SDL_SetColorKey(fontSurface, SDL_SRCCOLORKEY | SDL_RLEACCEL, 
	                SDL_MapRGB(fontSurface->format, r, g, b));
}

SDLFont::~SDLFont()
{
	SDL_FreeSurface(fontSurface);
}

void SDLFont::drawText(const std::string &string, SDL_Surface *surface, int x, int y)
{
	// see how many characters can fit on the screen
	if (x > surface->w || y > surface->h)
		return;
	int characters = string.length();
	if (characters > (surface->w - x) / charWidth)
		characters = (surface->w - x) / charWidth;

	SDL_Rect destRect;
	destRect.x = x;
	destRect.y = y;
	destRect.w = charWidth;
	destRect.h = charHeight;

	SDL_Rect sourceRect;
	sourceRect.w = charWidth;
	sourceRect.h = charHeight;

	// Now draw it
	for (int loop=0; loop<characters; loop++) {
		sourceRect.x = (string[loop] % CHARS_PER_ROW) * charWidth;
		sourceRect.y = (string[loop] / CHARS_PER_ROW) * charHeight;
		SDL_BlitSurface(fontSurface, &sourceRect, surface, &destRect);
		destRect.x += charWidth;
	}
}

int SDLFont::height()
{
	return charHeight;
}

int SDLFont::width()
{
	return charWidth;
}

