// $Id: MSXPPI.cc,v 1.47 2002/08/16 18:36:49 m9710797 Exp $

#include <cassert>
#include "MSXPPI.hh"
#include "Keyboard.hh"
#include "Leds.hh"
#include "MSXCPUInterface.hh"
#include "KeyClick.hh"
#include "CassettePort.hh"

// MSXDevice

MSXPPI::MSXPPI(MSXConfig::Device *config, const EmuTime &time)
	: MSXDevice(config, time), MSXIODevice(config, time)
{
	short volume = (short)deviceConfig->getParameterAsInt("volume");
	bool keyGhosting = deviceConfig->getParameterAsBool("key_ghosting");
	keyboard = new Keyboard(keyGhosting);
	i8255 = new I8255(*this, time);
	click = new KeyClick(volume, time);
	cpuInterface = MSXCPUInterface::instance();
	cassettePort = CassettePortFactory::instance(time);
	leds = Leds::instance();
	
	// Register I/O ports A8..AB for reading
	MSXCPUInterface::instance()->register_IO_In(0xA8,this);
	MSXCPUInterface::instance()->register_IO_In(0xA9,this);
	MSXCPUInterface::instance()->register_IO_In(0xAA,this);
	MSXCPUInterface::instance()->register_IO_In(0xAB,this);
	// Register I/O ports A8..AB for writing
	MSXCPUInterface::instance()->register_IO_Out(0xA8,this);
	MSXCPUInterface::instance()->register_IO_Out(0xA9,this); 
	MSXCPUInterface::instance()->register_IO_Out(0xAA,this);
	MSXCPUInterface::instance()->register_IO_Out(0xAB,this);

	reset(time);
}

MSXPPI::~MSXPPI()
{
	PRT_DEBUG("Destroying an MSXPPI object");
	delete keyboard;
	delete i8255;
	delete click;
}


void MSXPPI::reset(const EmuTime &time)
{
	i8255->reset(time);
	click->reset(time);
}

byte MSXPPI::readIO(byte port, const EmuTime &time)
{
	switch (port) {
		case 0xA8:
			return i8255->readPortA(time);
		case 0xA9:
			return i8255->readPortB(time);
		case 0xAA:
			return i8255->readPortC(time);
		case 0xAB:
			return i8255->readControlPort(time);
	}
	assert (false); // code should never be reached
	return 255;	// avoid warning
}

void MSXPPI::writeIO(byte port, byte value, const EmuTime &time)
{
	switch (port) {
		case 0xA8:
			i8255->writePortA(value, time);
			break;
		case 0xA9:
			i8255->writePortB(value, time);
			break;
		case 0xAA:
			i8255->writePortC(value, time);
			break;
		case 0xAB:
			i8255->writeControlPort(value, time);
			break;
	}
}


// I8255Interface

byte MSXPPI::readA(const EmuTime &time)
{
	// port A is normally an output on MSX, reading from an output port
	// is handled internally in the 8255
	return 255;	//TODO check this
}
void MSXPPI::writeA(byte value, const EmuTime &time)
{
	cpuInterface->setPrimarySlots(value);
}

byte MSXPPI::readB(const EmuTime &time)
{
	return keyboard->getKeys()[selectedRow];
}
void MSXPPI::writeB(byte value, const EmuTime &time)
{
	// probably nothing happens on a real MSX
}

nibble MSXPPI::readC1(const EmuTime &time)
{
	return 15;	// TODO check this
}
nibble MSXPPI::readC0(const EmuTime &time)
{
	return 15;	// TODO check this
}
void MSXPPI::writeC1(nibble value, const EmuTime &time)
{
	cassettePort->setMotor(!(value & 1), time);	// 0=0n, 1=Off
	cassettePort->cassetteOut(value & 2, time);
	
	Leds::LEDCommand caps = (value & 4) ? Leds::CAPS_OFF : Leds::CAPS_ON;
	Leds::instance()->setLed(caps);

	click->setClick(value & 8, time);
}
void MSXPPI::writeC0(nibble value, const EmuTime &time)
{
	selectedRow = value;
}
