// $Id: MSXPPI.hh,v 1.33 2002/08/16 18:36:49 m9710797 Exp $

// This class implements the PPI (8255)
//
//   PPI    MSX-I/O  Direction  MSX-Function
//  PortA    0xA8      Out     Memory primary slot register
//  PortB    0xA9      In      Keyboard column inputs
//  PortC    0xAA      Out     Keyboard row select / CAPS / CASo / CASm / SND
//  Control  0xAB     In/Out   Mode select for PPI
//
//  Direction indicates the direction normally used on MSX.
//  Reading from an output port returns the last written byte.
//  Writing to an input port has no immediate effect.
//
//  PortA combined with upper half of PortC form groupA
//  PortB               lower                    groupB
//  GroupA can be in programmed in 3 modes
//   - basic input/output
//   - strobed input/output
//   - bidirectional
//  GroupB can only use the first two modes.
//  Only the first mode is used on MSX, only this mode is implemented yet.
//
//  for more detail see
//    http://w3.qahwah.net/joost/openMSX/8255.pdf

#ifndef __MSXPPI_HH__
#define __MSXPPI_HH__

#include "MSXIODevice.hh"
#include "I8255.hh"

// forward declarations
class KeyClick;
class MSXCPUInterface;
class CassettePortInterface;
class Leds;
class Keyboard;


class MSXPPI : public MSXIODevice, public I8255Interface
{
	// MSXDevice
	public:
		/**
		 * Constructor.
		 */
		MSXPPI(MSXConfig::Device *config, const EmuTime &time); 

		/**
		 * Destructor
		 */
		virtual ~MSXPPI(); 

		virtual void reset(const EmuTime &time);
		virtual byte readIO(byte port, const EmuTime &time);
		virtual void writeIO(byte port, byte value, const EmuTime &time);

	private:
		I8255 *i8255;
	
	// I8255Interface
	public:
		virtual byte readA(const EmuTime &time);
		virtual byte readB(const EmuTime &time);
		virtual nibble readC0(const EmuTime &time);
		virtual nibble readC1(const EmuTime &time);
		virtual void writeA(byte value, const EmuTime &time);
		virtual void writeB(byte value, const EmuTime &time);
		virtual void writeC0(nibble value, const EmuTime &time);
		virtual void writeC1(nibble value, const EmuTime &time);
	
	private:
		MSXCPUInterface *cpuInterface;
		CassettePortInterface *cassettePort;
		Leds *leds;
		KeyClick *click;
		Keyboard *keyboard;
		nibble selectedRow;
};
#endif
