// $Id: EventDistributor.hh,v 1.5 2002/08/03 19:23:47 m9710797 Exp $

#ifndef __EVENTDISTRIBUTOR_HH__
#define __EVENTDISTRIBUTOR_HH__

#include <SDL/SDL.h>
#include <map>
#include <queue>
#include "Thread.hh"
#include "Mutex.hh"
#include "Schedulable.hh"

// forward declaration
class EventListener;


class EventDistributor : public Runnable , public Schedulable
{
	public:
		virtual ~EventDistributor();
		static EventDistributor *instance();

		/**
		 * Use this method to (un)register a given class to receive
		 * certain SDLEvents. 
		 * @param type The SDLEventType number of the events you want to receive
		 * @param listener Object that will be notified when the events arrives
		 * @param priority The priority of the listener (lower number is higher
		 *        priority). Higher priority listeners may block an event for
		 *        lower priority listeners. Normally you don't need to specify
		 *        a priority.
		 *        Note: in the current implementation there are only two
		 *              priority levels (0 and !=0)
		 * The delivery of the event is done by the 'main-emulation-thread',
		 * so there is no need for extra synchronization.
		 */
		void   registerEventListener(int type, EventListener *listener, int priority = 0);
		void unregisterEventListener(int type, EventListener *listener, int priority = 0);
		
	private:
		EventDistributor();
		virtual void executeUntilEmuTime(const EmuTime &time, int userdata);
		virtual const std::string &schedName();
		virtual void run();

		std::multimap <int, EventListener*> lowMap;
		std::multimap <int, EventListener*> highMap;
		std::queue <std::pair<SDL_Event, EventListener*> > lowQueue;
		std::queue <std::pair<SDL_Event, EventListener*> > highQueue;
		Mutex mutex;
};

#endif
