// $Id: MSXAudio.cc,v 1.7 2002/08/29 13:09:48 m9710797 Exp $

#include "MSXAudio.hh"
#include "MSXCPUInterface.hh"
#include "Mixer.hh"
#include "Y8950.hh"


MSXAudio::MSXAudio(MSXConfig::Device *config, const EmuTime &time)
	: MSXDevice(config, time), MSXIODevice(config, time)
{
	MSXCPUInterface::instance()->register_IO_Out(0xc0, this);
	MSXCPUInterface::instance()->register_IO_Out(0xc1, this);
	MSXCPUInterface::instance()->register_IO_In (0xc0, this);
	MSXCPUInterface::instance()->register_IO_In (0xc1, this);
	short volume = (short)deviceConfig->getParameterAsInt("volume");
	
	// left / right / mono
	Mixer::ChannelMode mode = Mixer::MONO;
	try {
		std::string stereomode = config->getParameter("mode");
		if      (stereomode == "left")
			mode=Mixer::MONO_LEFT;
		else if (stereomode == "right")
			mode=Mixer::MONO_RIGHT;
	} catch (MSXException &e) {
		PRT_ERROR("Exception: " << e.desc);
	}
	
	// SampleRAM size
	int ramSize = 256;	// in kb
	try {
		ramSize = config->getParameterAsInt("sampleram");
	} catch (MSXException &e) {
		//  no size specified
	}
	
	y8950 = new Y8950(volume, ramSize * 1024, time, mode);
	reset(time);
}

MSXAudio::~MSXAudio()
{
	delete y8950;
}

void MSXAudio::reset(const EmuTime &time)
{
	y8950->reset(time);
	registerLatch = 0;	// TODO check
}

byte MSXAudio::readIO(byte port, const EmuTime &time)
{
	byte result;
	switch (port) {
		case 0xc0:
			result = y8950->readStatus();
			break;
		case 0xc1:
			result = y8950->readReg(registerLatch, time);
			break;
		default:
			assert(false);
			result = 0;	// avoid warning
	}
	PRT_DEBUG("Audio: read "<<std::hex<<(int)port<<" "<<(int)result<<std::dec);
	return result;
}

void MSXAudio::writeIO(byte port, byte value, const EmuTime &time)
{
	switch (port) {
		case 0xc0:
			registerLatch = value;
			break;
		case 0xc1:
			y8950->writeReg(registerLatch, value, time);
			break;
	}
}
